//==============================================================================
// ArduinoPrint - Add Arduino Print type objects.  Both for hardware serial ports
//		as well as for bit bang serial.
//
// Version 0.1 - Work in Progress
//
// Original Print.cpp - Base class that provides print() and println()
// Copyright (c) 2008 David A. Mellis.  All right reserved.
//
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#include "BapLib.h"
#include <stdarg.h>
#include "hsio.h"

//==============================================================================
// C++ Print wrappers to emulate Arduino print functinos...
//==============================================================================


void Print::setWriteCB(PWRITECB pwritecb)
{
	_pwritecb = pwritecb;
}

void Print::write(byte b)
{
	if (_pwritecb) {
		(*_pwritecb)(this, b);
	}
}


// Now include C++ functions for semi-arduino support...
void Print::write(const char *str)
{
	while (*str)
    	write(*str++);
}

/* default implementation: may be overridden */
void Print::write(const byte *buffer, byte size)
{
	while (size--)
    	write(*buffer++);
}

void Print::print(const char str[])
{
	write(str);
}

void Print::print(char c, int base)
{
	print((long) c, base);
}

void Print::print(unsigned char b, int base)
{
	print((unsigned long) b, base);
}

void Print::print(int n, int base)
{
	print((long) n, base);
}

void Print::print(unsigned int n, int base)
{
	print((unsigned long) n, base);
}

void Print::print(long n, int base)
{
	if (base == 0) {
    	write(n);
  	} else if (base == 10) {
    	if (n < 0) {
      		print('-');
      		n = -n;
    	}
    	printNumber(n, 10);
  	} else {
    	printNumber(n, base);
  	}
}

void Print::print(unsigned long n, int base)
{
	if (base == 0) write(n);
	else printNumber(n, base);
}

void Print::println(void)
{
	print('\r');
	print('\n');  
}

void Print::println(const char c[])
{
	print(c);
	println();
}

void Print::println(char c, int base)
{
	print(c, base);
	println();
}

void Print::println(unsigned char b, int base)
{
	print(b, base);
	println();
}

void Print::println(int n, int base)
{
	print(n, base);
	println();
}

void Print::println(unsigned int n, int base)
{
	print(n, base);
	println();
}

void Print::println(long n, int base)
{
	print(n, base);
	println();
}

void Print::println(unsigned long n, int base)
{
	print(n, base);
	println();
}

// Private Methods /////////////////////////////////////////////////////////////

void Print::printNumber(unsigned long n, byte base)
{
	unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
	unsigned long i = 0;
	
	if (n == 0) {
		print('0');
		return;
	} 
	
	while (n > 0) {
		buf[i++] = n % base;
		n /= base;
	}
	
	for (; i > 0; i--)
		print((char) (buf[i - 1] < 10 ?
  			'0' + buf[i - 1] :
  			'A' + buf[i - 1] - 10));
}


void HardwareSerialCB(Print *p, byte b)
{
	hserout(((HardwareSerial*)p)->_bWhich, &b, 1);
}


HardwareSerial::HardwareSerial(byte bWhich)
{
	_bWhich = bWhich;
	setWriteCB((PWRITECB)&HardwareSerialCB);
	enablehserial(_bWhich);
}


void HardwareSerial::begin(long lBaud)
{
	sethserial (_bWhich, lBaud);
}

void HardwareSerial::end()
{
}

byte HardwareSerial::available(void)
{
	return hseravail(_bWhich);
}

int HardwareSerial::read(void)
{
	u8 b;		// byte to read in...
	if (hserin(_bWhich, 0, &b, 1, 0xffff) != 0)
		return (int)b;
	return -1;
}

void HardwareSerial::flush(void)
{
	hserstat(_bWhich, HSERSTAT_CLEAR_INPUT_BUFFER);
}

void HardwareSerial::write(byte b)
{
	hserout(_bWhich, &b, 1);
}



void SoftwareSerialCB(Print *p, byte b)
{
	SoftwareSerial *pss = (SoftwareSerial*)p;
	if ((pss->_bPinOut & 0x80) == 0)	// should handle all special case ports...
		TASerout(pss->_bPinOut, pss->_wMode, (char*)&b, 1);
}



SoftwareSerial::SoftwareSerial(byte bPinIn, byte bPinOut)
{
	// Pass in 0xff if not desired
	_bPinIn = bPinIn;
	_bPinOut = bPinOut;

}

void SoftwareSerial::begin(word wMode)
{
	if (_bPinIn != 0xff)
		input(_bPinIn);	// make sure set to input pin
	if (_bPinOut != 0xff)
		output(_bPinOut);
		
	_wMode = wMode;
}

void SoftwareSerial::end()
{
}

byte SoftwareSerial::available(void)
{
	// Not sure what to do, need some form of timeout...?	
	return 0xff;	//???
}

int SoftwareSerial::read(unsigned long lTimeout)
{
	// No buffer so nothing to flush
	byte b;
	if (TASerin(_bPinIn, 0xff, _wMode, (char*)&b, 1, lTimeout, 0xffff) != 0)
		return (int)b;
	return -1;
	
}


void SoftwareSerial::flush(void)
{
}

void SoftwareSerial::write(const byte *buffer, byte size)
{
	if (_bPinOut != 0xff)
		TASerout(_bPinOut, _wMode, (char*)buffer, size);
}


void SoftwareSerial::write(byte b)
{
	if (_bPinOut != 0xff)
		TASerout(_bPinOut, _wMode, (char*)&b, 1);
}

//==================================================================================================
HPrint2 hp2(1);
SPrint2 sp2();

void Print2::Write(const byte *buffer, byte size)
{
	;
}

void HPrint2::Write(byte b) 
{
	;
}

HPrint2::HPrint2(byte bPin)
{
	;
}	

void SPrint2::Write(byte b)
{
	;
}

SPrint2::SPrint2()
{
	;		
}
